/* Turret Plug-in v2.0
 * Made by Yassen
 * Source and binaries can be found on the BZBB (http://my.bzflag.org/bb) under Plug-in Releases
 */

#include "bzfsAPI.h"
#include <vector>
#include <string>
#include <map>
#include <stdio.h> // For stupid linux
#include <stdlib.h>
#include <string.h>
#include "TurretWeapon.h"
#include "TurretEjector.h"
#include "TurretZone.h"
#include "common.h"
#include "ShotTracker.h"



extern unsigned int debugLevel;
extern _ShotTracker ShotTracker;
std::vector<TurretWeapon> turretWeaponList;
std::vector<TurretZone> turretZoneList;
std::vector<TurretEjector> turretEjectorList;
bool isPlaying [256] = {true};
double lastTime = 0.0;
int frequency = -1;
double tickNumber = 0.0;

void sendHelpMessage(){

	bz_debugMessage(0, "Turret - ======================================================================");
	bz_debugMessage(0, "Turret - == Command Line HELP ==");
	bz_debugMessage(0, "Turret - = Loading =");
	bz_debugMessage(0, "Turret - Syntax: '-loadplugin path/to/Turret,<debugLevel>");
	bz_debugMessage(0, "Turret - = Debugging =");
	bz_debugMessage(0, "Turret - Debugging is disabled by default. To enable, pass a number (0-4) as");
	bz_debugMessage(0, "Turret - the loading parameter.");
	bz_debugMessage(0, "Turret - = Debugging Values =");
	bz_debugMessage(0, "Turret - 0 - No debugging (default)");
	bz_debugMessage(0, "Turret - 1 - Shows ERROR messages. These messages are shown when something");
	bz_debugMessage(0, "Turret -     happens that the plug-in can't recover from.");
	bz_debugMessage(0, "Turret - 2 - Shows WARNING messages. These messages are shown when something");
	bz_debugMessage(0, "Turret -     happens that is recoverable, but will give unexpected results.");
	bz_debugMessage(0, "Turret - 3 - Shows NOTICE messages. These messages are shown when something");
	bz_debugMessage(0, "Turret -     very unimportant happens.");
	bz_debugMessage(0, "Turret - 4 - Shows DEBUG messages. These messages are shown for help with");
	bz_debugMessage(0, "Turret -     debugging.");
	bz_debugMessage(0, "Turret - Recommended debugging level: 2");
	bz_debugMessage(0, "Turret - ======================================================================");

}

class TurretMapHandler : public bz_CustomMapObjectHandler
{
public:
	virtual bool MapObject ( bz_ApiString object, bz_CustomMapObjectInfo *data );
};

TurretMapHandler	turretmaphandler;

class TurretHandler : public bz_Plugin
{
public:
	virtual const char* Name (){return "Turret Plugin";}
	virtual void Init ( const char* config)
	{
		Register(bz_ePlayerDieEvent);
		Register(bz_ePlayerJoinEvent);
		Register(bz_ePlayerPartEvent);
		Register(bz_ePlayerPausedEvent);
		Register(bz_eShotEndedEvent);
		Register(bz_eTickEvent);

		bz_registerCustomMapObject("turretWeapon", &turretmaphandler);
		bz_registerCustomMapObject("turretEjector", &turretmaphandler);
		bz_registerCustomMapObject("turretZone", &turretmaphandler);

		turretWeaponList.clear();
		turretEjectorList.clear();
		turretZoneList.clear();

		if(strlen(config) > 1)
			sendHelpMessage();

		if(strcmp(config, "") == 0){
			sendHelpMessage();
			debugLevel = 0;
		}

		else
			debugLevel = atoi(config);

		if(debugLevel > 4){
			sendHelpMessage();
			debugLevel = 4;
		}
	}
	virtual void Event(bz_EventData *eventData);
	virtual void Cleanup ();
};

BZ_PLUGIN(TurretHandler);

/*
BZF_PLUGIN_CALL int Init(const char* commandLine)
{
	bz_registerEvent(bz_ePlayerDieEvent);
	bz_registerEvent(bz_ePlayerJoinEvent);
	bz_registerEvent(bz_ePlayerPartEvent);
	bz_registerEvent(bz_ePlayerPausedEvent);
	bz_registerEvent(bz_eShotEndedEvent);
	bz_registerEvent(bz_eTickEvent);

	bz_registerCustomMapObject("turretWeapon", &Turret);
	bz_registerCustomMapObject("turretEjector", &Turret);
	bz_registerCustomMapObject("turretZone", &Turret);

	turretWeaponList.clear();
	turretEjectorList.clear();
	turretZoneList.clear();

	if(strlen(commandLine) > 1)
		sendHelpMessage();

	if(strcmp(commandLine, "") == 0){
		sendHelpMessage();
		debugLevel = 0;
	}

	else
		debugLevel = atoi(commandLine);

	if(debugLevel > 4){
		sendHelpMessage();
		debugLevel = 4;
	}

	return 0;
}
*/



void TurretHandler::Event(bz_EventData *eventData)
{
	switch (eventData->eventType)
	{
		case bz_ePlayerDieEvent: //A player dies
		{
			bz_PlayerDieEventData_V1* data = (bz_PlayerDieEventData_V1*)eventData;

#ifdef DEBUG
			bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "%s killed by %s with shot ID of %i (killer ID: %i", getCallsign(data->playerID).c_str(), getCallsign(data->killerID).c_str(), data->shotID, data->killerID);
#endif

			if(data->killerID == 253)
			{
				Shot shot = ShotTracker.findShot(data->shotID);
				if(!shot.isEmpty)
				{
					data->killerID = shot.owner;
				}
			}

			for(unsigned int i = 0; i < turretZoneList.size(); i++)
				if(turretZoneList[i].getOccupent() == data->playerID)
					turretZoneList[i].setOccupent(-1);
		}
		break;

		case bz_ePlayerJoinEvent:
		{
			bz_PlayerJoinPartEventData_V1* data = (bz_PlayerJoinPartEventData_V1*)eventData;

			isPlaying[data->playerID] = true;
		}
		break;

		case bz_ePlayerPartEvent:
		{
			bz_PlayerJoinPartEventData_V1* data = (bz_PlayerJoinPartEventData_V1*)eventData;

			isPlaying[data->playerID] = false;

			for(std::vector<TurretZone>::iterator it = turretZoneList.begin(); it != turretZoneList.end(); it++)
				if(it->getOccupent() == data->playerID)
					it->setOccupent(-1);
		}

		case bz_ePlayerPausedEvent:
		{
			bz_PlayerPausedEventData_V1* data = (bz_PlayerPausedEventData_V1*)eventData;

			if(data->pause)
			{
				for(std::vector<TurretZone>::iterator it = turretZoneList.begin(); it != turretZoneList.end(); it++)
					if(it->getOccupent() == data->playerID)
						it->setOccupent(-1);

				isPlaying[data->playerID] = false;
			}
			
			else
				isPlaying[data->playerID] = true;
		}
		break;

		case bz_eShotEndedEvent:
		{
			bz_ShotEndedEventData_V1* data = (bz_ShotEndedEventData_V1*)eventData;

#ifdef DEBUG
			bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "Ending shot %i", data->shotID);
#endif

			ShotTracker.remove(data->shotID);
		}

		case bz_eTickEvent: //The server ticks
		{
			bz_TickEventData_V1* data = (bz_TickEventData_V1*)eventData;

			if(tickNumber == 0)
			{
				for(std::vector<TurretWeapon>::iterator it = turretWeaponList.begin(); it != turretWeaponList.end(); it++)
					if(it->getFrequency() > frequency)
						frequency = it->getFrequency();

				if(frequency < 10)
					frequency = 10; // So we don't get too low, 10 times a second shouldn't cause too many problems.

#ifdef DEBUG
				bz_debugMessagef(0, "Checking %i times a second.", frequency);
#endif
			}

			if((data->eventTime - lastTime) > (1.0/frequency))
			{
#ifdef DEBUG
				bz_debugMessagef(0, "Ticking at %f", (float)data->eventTime);
#endif

				bz_APIIntList *playerlist = bz_newIntList();
				bz_getPlayerIndexList(playerlist);

				for(unsigned int i = 0; i < playerlist->size(); i++)
				{
					bz_BasePlayerRecord *player = bz_getPlayerByIndex(playerlist->get(i));

					if(!player || !player->spawned || !isPlaying[player->playerID])
					{
						bz_freePlayerRecord(player);
						continue;
					}

					for(std::vector<TurretZone>::iterator it = turretZoneList.begin(); it != turretZoneList.end(); it++)
						it->checkPosition(player);

					for(std::vector<TurretEjector>::iterator it = turretEjectorList.begin(); it != turretEjectorList.end(); it++)
						it->checkPosition(player);

					bz_freePlayerRecord(player);
				}

				ShotTracker.purge();

				lastTime -= 1.0/frequency;
			}

			tickNumber++;
		}
	}
}

bool TurretMapHandler::MapObject(bz_ApiString object, bz_CustomMapObjectInfo *data){

	sendDebugMessage((std::string)bz_format(" ---- Handling object: %s ---- ", object.c_str()));
	if(object == "TURRETZONE")
	{
		TurretZone turret = TurretZone(&turretWeaponList);
		if(turret.parse(data))
			turretZoneList.push_back(turret);
	}

	if(object == "TURRETWEAPON")
	{
		TurretWeapon turret = TurretWeapon();
		if(turret.parse(data))
			turretWeaponList.push_back(turret);
	}
	else if(object == "TURRETEJECTOR")
	{
		TurretEjector turret = TurretEjector(&turretZoneList);
		if(turret.parse(data))
			turretEjectorList.push_back(turret);
	}
	return true;
}

void TurretHandler::Cleanup ( void )
{
	Flush();
	bz_removeCustomMapObject("turretweapon");
	bz_removeCustomMapObject("turretejector");
	bz_removeCustomMapObject("turretzone");

	turretEjectorList.clear();
	turretWeaponList.clear();
	turretZoneList.clear();

	bz_debugMessage(4,"Turret plugin unloaded");
}

/*
BZF_PLUGIN_CALL int bz_Unload()
{
	bz_removeEvent(bz_ePlayerDieEvent, &Turret);
	bz_removeEvent(bz_ePlayerJoinEvent, &Turret);
	bz_removeEvent(bz_ePlayerPartEvent, &Turret);
	bz_removeEvent(bz_ePlayerPausedEvent, &Turret);
	bz_removeEvent(bz_eShotEndedEvent, &Turret);
	bz_removeEvent(bz_eTickEvent, &Turret);

	bz_removeCustomMapObject("turretweapon");
	bz_removeCustomMapObject("turretejector");
	bz_removeCustomMapObject("turretzone");

	turretEjectorList.clear();
	turretWeaponList.clear();
	turretZoneList.clear();

	bz_debugMessage(4,"Turret plugin unloaded");
	return 0;

}
*/

// Local Variables: ***
// mode:C++ ***
// tab-width: 8 ***
// c-basic-offset: 2 ***
// indent-tabs-mode: t ***
// End: ***
// ex: shiftwidth=2 tabstop=8
